pub struct Bytecode(pub Bytecode);Expand description
Bytecode for an account.
A wrapper around revm::primitives::Bytecode with encoding/decoding support.
Tuple Fields
0: BytecodeImplementations
Sourceimpl Bytecode
impl Bytecode
Sourcepub fn new_raw(bytes: Bytes) -> Self
pub fn new_raw(bytes: Bytes) -> Self
Create new bytecode from raw bytes.
No analysis will be performed.
Panics
Panics if bytecode is EOF and has incorrect format.
Sourcepub fn new_raw_checked(bytecode: Bytes) -> Result<Self, BytecodeDecodeError>
pub fn new_raw_checked(bytecode: Bytes) -> Result<Self, BytecodeDecodeError>
Creates a new raw revm_bytecode::Bytecode.
Returns an error on incorrect Bytecode format.
Methods from Deref<Target = RevmBytecode>
Sourcepub fn kind(&self) -> BytecodeKind
pub fn kind(&self) -> BytecodeKind
Returns the kind of bytecode.
Sourcepub fn is_eip7702(&self) -> bool
pub fn is_eip7702(&self) -> bool
Returns true if bytecode is EIP-7702.
Sourcepub fn eip7702_address(&self) -> Option<Address>
pub fn eip7702_address(&self) -> Option<Address>
Returns the EIP-7702 delegated address if this is EIP-7702 bytecode.
Sourcepub fn legacy_jump_table(&self) -> Option<&JumpTable>
pub fn legacy_jump_table(&self) -> Option<&JumpTable>
Returns jump table if bytecode is legacy analyzed.
Sourcepub fn hash_slow(&self) -> FixedBytes<32>
pub fn hash_slow(&self) -> FixedBytes<32>
Calculates or returns cached hash of the bytecode.
Sourcepub fn bytecode(&self) -> &Bytes
pub fn bytecode(&self) -> &Bytes
Returns a reference to the bytecode bytes.
For legacy bytecode, this includes padding. For EIP-7702, this is the raw bytes.
Sourcepub fn bytecode_ptr(&self) -> *const u8
pub fn bytecode_ptr(&self) -> *const u8
Pointer to the bytecode bytes.
Sourcepub fn bytes_slice(&self) -> &[u8] ⓘ
pub fn bytes_slice(&self) -> &[u8] ⓘ
Returns the bytecode as a slice.
Sourcepub fn original_bytes(&self) -> Bytes
pub fn original_bytes(&self) -> Bytes
Returns the original bytecode without padding.
Sourcepub fn original_byte_slice(&self) -> &[u8] ⓘ
pub fn original_byte_slice(&self) -> &[u8] ⓘ
Returns the original bytecode as a byte slice without padding.
Sourcepub fn iter_opcodes(&self) -> BytecodeIterator<'_>
pub fn iter_opcodes(&self) -> BytecodeIterator<'_>
Returns an iterator over the opcodes in this bytecode, skipping immediates.