pub struct RecoveredBlock<B: Block> { /* private fields */ }Expand description
A block with senders recovered from the block’s transactions.
This type represents a SealedBlock where all transaction senders have been
recovered and verified. Recovery is an expensive operation that extracts the
sender address from each transaction’s signature.
Construction
RecoveredBlock::new/RecoveredBlock::new_unhashed- Create with pre-recovered senders (unchecked)RecoveredBlock::try_new/RecoveredBlock::try_new_unhashed- Create with validationRecoveredBlock::try_recover- Recover from a blockRecoveredBlock::try_recover_sealed- Recover from a sealed block
Performance
Sender recovery is computationally expensive. Cache recovered blocks when possible to avoid repeated recovery operations.
Sealing
This type uses lazy sealing to avoid hashing the header until it is needed:
RecoveredBlock::new_unhashed creates a recovered block without hashing the header.
RecoveredBlock::new creates a recovered block with the corresponding block hash.
Recovery
Sender recovery is fallible and can fail if any of the transactions fail to recover the sender.
A SealedBlock can be upgraded to a RecoveredBlock using the
RecoveredBlock::try_recover or SealedBlock::try_recover method.
Implementations
Sourceimpl<B: Block> RecoveredBlock<B>
impl<B: Block> RecoveredBlock<B>
Sourcepub fn new(block: B, senders: Vec<Address>, hash: BlockHash) -> Self
pub fn new(block: B, senders: Vec<Address>, hash: BlockHash) -> Self
Creates a new recovered block instance with the given senders as provided and the block hash.
Note: This expects that the given senders match the transactions in the block.
Sourcepub fn new_unhashed(block: B, senders: Vec<Address>) -> Self
pub fn new_unhashed(block: B, senders: Vec<Address>) -> Self
Creates a new recovered block instance with the given senders as provided.
Note: This expects that the given senders match the transactions in the block.
Sourcepub fn senders_iter(&self) -> impl Iterator<Item = &Address>
pub fn senders_iter(&self) -> impl Iterator<Item = &Address>
Returns an iterator over the recovered senders.
Sourcepub fn into_block(self) -> B
pub fn into_block(self) -> B
Consumes the type and returns the inner block.
Sourcepub const fn sealed_block(&self) -> &SealedBlock<B>
pub const fn sealed_block(&self) -> &SealedBlock<B>
Returns a reference to the sealed block.
Sourcepub const fn new_sealed(block: SealedBlock<B>, senders: Vec<Address>) -> Self
pub const fn new_sealed(block: SealedBlock<B>, senders: Vec<Address>) -> Self
Creates a new recovered block instance with the given SealedBlock and senders as
provided
Sourcepub fn try_new(
block: B,
senders: Vec<Address>,
hash: BlockHash,
) -> Result<Self, SealedBlockRecoveryError<B>>
pub fn try_new( block: B, senders: Vec<Address>, hash: BlockHash, ) -> Result<Self, SealedBlockRecoveryError<B>>
A safer variant of Self::new that checks if the number of senders is equal to
the number of transactions in the block and recovers the senders from the transactions, if
not using SignedTransaction::recover_signer
to recover the senders.
Sourcepub fn try_new_unchecked(
block: B,
senders: Vec<Address>,
hash: BlockHash,
) -> Result<Self, SealedBlockRecoveryError<B>>
pub fn try_new_unchecked( block: B, senders: Vec<Address>, hash: BlockHash, ) -> Result<Self, SealedBlockRecoveryError<B>>
A safer variant of Self::new that checks if the number of senders is equal to
the number of transactions in the block and recovers the senders from the transactions, if
not using SignedTransaction::recover_signer_unchecked
to recover the senders.
Sourcepub fn try_new_unhashed(
block: B,
senders: Vec<Address>,
) -> Result<Self, RecoveryError>
pub fn try_new_unhashed( block: B, senders: Vec<Address>, ) -> Result<Self, RecoveryError>
A safer variant of Self::new_unhashed that checks if the number of senders is equal to
the number of transactions in the block and recovers the senders from the transactions, if
not using SignedTransaction::recover_signer
to recover the senders.
Sourcepub fn try_new_unhashed_unchecked(
block: B,
senders: Vec<Address>,
) -> Result<Self, RecoveryError>
pub fn try_new_unhashed_unchecked( block: B, senders: Vec<Address>, ) -> Result<Self, RecoveryError>
A safer variant of Self::new_unhashed that checks if the number of senders is equal to
the number of transactions in the block and recovers the senders from the transactions, if
not using SignedTransaction::recover_signer_unchecked
to recover the senders.
Sourcepub fn try_recover(block: B) -> Result<Self, RecoveryError>
pub fn try_recover(block: B) -> Result<Self, RecoveryError>
Recovers the senders from the transactions in the block using
SignedTransaction::recover_signer.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn try_recover_unchecked(block: B) -> Result<Self, RecoveryError>
pub fn try_recover_unchecked(block: B) -> Result<Self, RecoveryError>
Recovers the senders from the transactions in the block using
SignedTransaction::recover_signer_unchecked.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn try_recover_sealed(
block: SealedBlock<B>,
) -> Result<Self, SealedBlockRecoveryError<B>>
pub fn try_recover_sealed( block: SealedBlock<B>, ) -> Result<Self, SealedBlockRecoveryError<B>>
Recovers the senders from the transactions in the block using
SignedTransaction::recover_signer.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn try_recover_sealed_unchecked(
block: SealedBlock<B>,
) -> Result<Self, SealedBlockRecoveryError<B>>
pub fn try_recover_sealed_unchecked( block: SealedBlock<B>, ) -> Result<Self, SealedBlockRecoveryError<B>>
Recovers the senders from the transactions in the sealed block using
SignedTransaction::recover_signer_unchecked.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn try_recover_sealed_with_senders(
block: SealedBlock<B>,
senders: Vec<Address>,
) -> Result<Self, SealedBlockRecoveryError<B>>
pub fn try_recover_sealed_with_senders( block: SealedBlock<B>, senders: Vec<Address>, ) -> Result<Self, SealedBlockRecoveryError<B>>
A safer variant of Self::new_sealed that checks if the number of senders is equal to
the number of transactions in the block and recovers the senders from the transactions, if
not using SignedTransaction::recover_signer
to recover the senders.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn try_recover_sealed_with_senders_unchecked(
block: SealedBlock<B>,
senders: Vec<Address>,
) -> Result<Self, SealedBlockRecoveryError<B>>
pub fn try_recover_sealed_with_senders_unchecked( block: SealedBlock<B>, senders: Vec<Address>, ) -> Result<Self, SealedBlockRecoveryError<B>>
A safer variant of Self::new_sealed that checks if the number of senders is equal to
the number of transactions in the block and recovers the senders from the transactions, if
not using SignedTransaction::recover_signer_unchecked
to recover the senders.
Sourcepub fn num_hash(&self) -> BlockNumHash
pub fn num_hash(&self) -> BlockNumHash
Return the number hash tuple.
Sourcepub fn block_with_parent(&self) -> BlockWithParent
pub fn block_with_parent(&self) -> BlockWithParent
Return a BlockWithParent for this header.
Sourcepub fn clone_header(&self) -> B::Header
pub fn clone_header(&self) -> B::Header
Clone the header.
Sourcepub fn clone_sealed_header(&self) -> SealedHeader<B::Header>
pub fn clone_sealed_header(&self) -> SealedHeader<B::Header>
Clones the internal header and returns a SealedHeader sealed with the hash.
Sourcepub fn clone_sealed_block(&self) -> SealedBlock<B>
pub fn clone_sealed_block(&self) -> SealedBlock<B>
Clones the wrapped block and returns the SealedBlock sealed with the hash.
Sourcepub fn into_header(self) -> B::Header
pub fn into_header(self) -> B::Header
Consumes the block and returns the block’s header.
Sourcepub fn into_sealed_block(self) -> SealedBlock<B>
pub fn into_sealed_block(self) -> SealedBlock<B>
Consumes the block and returns the SealedBlock and drops the recovered senders.
Sourcepub fn split_sealed(self) -> (SealedBlock<B>, Vec<Address>)
pub fn split_sealed(self) -> (SealedBlock<B>, Vec<Address>)
Consumes the type and returns its components.
Sourcepub fn recovered_transaction(
&self,
idx: usize,
) -> Option<Recovered<&<B::Body as BlockBody>::Transaction>>
pub fn recovered_transaction( &self, idx: usize, ) -> Option<Recovered<&<B::Body as BlockBody>::Transaction>>
Returns the Recovered<&T> transaction at the given index.
Sourcepub fn find_indexed(&self, tx_hash: TxHash) -> Option<IndexedTx<'_, B>>
pub fn find_indexed(&self, tx_hash: TxHash) -> Option<IndexedTx<'_, B>>
Finds a transaction by hash and returns it with its index and block context.
Sourcepub fn transactions_with_sender(
&self,
) -> impl Iterator<Item = (&Address, &<B::Body as BlockBody>::Transaction)> + '_
pub fn transactions_with_sender( &self, ) -> impl Iterator<Item = (&Address, &<B::Body as BlockBody>::Transaction)> + '_
Returns an iterator over all transactions and their sender.
Sourcepub fn clone_transactions_recovered(
&self,
) -> impl Iterator<Item = Recovered<<B::Body as BlockBody>::Transaction>> + '_
pub fn clone_transactions_recovered( &self, ) -> impl Iterator<Item = Recovered<<B::Body as BlockBody>::Transaction>> + '_
Returns an iterator over cloned Recovered<Transaction>
Sourcepub fn transactions_recovered(
&self,
) -> impl Iterator<Item = Recovered<&<B::Body as BlockBody>::Transaction>> + '_
pub fn transactions_recovered( &self, ) -> impl Iterator<Item = Recovered<&<B::Body as BlockBody>::Transaction>> + '_
Returns an iterator over Recovered<&Transaction>
Sourcepub fn into_transactions_recovered(
self,
) -> impl Iterator<Item = Recovered<<B::Body as BlockBody>::Transaction>>
pub fn into_transactions_recovered( self, ) -> impl Iterator<Item = Recovered<<B::Body as BlockBody>::Transaction>>
Consumes the type and returns an iterator over all Recovered transactions in the block.
Sourcepub fn into_transactions(self) -> Vec<<B::Body as BlockBody>::Transaction>
pub fn into_transactions(self) -> Vec<<B::Body as BlockBody>::Transaction>
Consumes the block and returns the transactions of the block.
Methods from Deref<Target = SealedBlock<B>>
Sourcepub fn clone_block(&self) -> B
pub fn clone_block(&self) -> B
Clones the wrapped block.
Sourcepub fn rlp_length(&self) -> usize
pub fn rlp_length(&self) -> usize
Returns the length of the block.
Sourcepub fn senders(&self) -> Result<Vec<Address>, RecoveryError>
pub fn senders(&self) -> Result<Vec<Address>, RecoveryError>
Recovers all senders from the transactions in the block.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn num_hash(&self) -> BlockNumHash
pub fn num_hash(&self) -> BlockNumHash
Return the number hash tuple.
Sourcepub fn block_with_parent(&self) -> BlockWithParent
pub fn block_with_parent(&self) -> BlockWithParent
Return a BlockWithParent for this header.
Sourcepub fn sealed_header(&self) -> &SealedHeader<B::Header>
pub fn sealed_header(&self) -> &SealedHeader<B::Header>
Returns the Sealed header.
Sourcepub fn sealed_header_ref(&self) -> SealedHeader<&B::Header>
pub fn sealed_header_ref(&self) -> SealedHeader<&B::Header>
Returns the wrapped SealedHeader<B::Header> as SealedHeader<&B::Header>.
Sourcepub fn clone_sealed_header(&self) -> SealedHeader<B::Header>
pub fn clone_sealed_header(&self) -> SealedHeader<B::Header>
Clones the wrapped header and returns a SealedHeader sealed with the hash.
Sourcepub fn blob_versioned_hashes_iter(&self) -> impl Iterator<Item = &B256> + '_
pub fn blob_versioned_hashes_iter(&self) -> impl Iterator<Item = &B256> + '_
Returns an iterator over all blob versioned hashes from the block body.
Sourcepub fn transaction_count(&self) -> usize
pub fn transaction_count(&self) -> usize
Returns the number of transactions in the block.
Sourcepub fn ensure_transaction_root_valid(&self) -> Result<(), GotExpected<B256>>
pub fn ensure_transaction_root_valid(&self) -> Result<(), GotExpected<B256>>
Ensures that the transaction root in the block header is valid.
The transaction root is the Keccak 256-bit hash of the root node of the trie structure populated with each transaction in the transactions list portion of the block.
Returns
Returns Ok(()) if the calculated transaction root matches the one stored in the header,
indicating that the transactions in the block are correctly represented in the trie.
Returns Err(error) if the transaction root validation fails, providing a GotExpected
error containing the calculated and expected roots.