pub struct SealedBlockWith<B: Block, T> { /* private fields */ }Expand description
A SealedBlock paired with associated data.
This is useful for workflows that need to carry block-adjacent metadata alongside a sealed block without defining a new wrapper type for each metadata payload.
Implementations
Sourceimpl<B: Block, T> SealedBlockWith<B, T>
impl<B: Block, T> SealedBlockWith<B, T>
Sourcepub const fn new(block: SealedBlock<B>, data: T) -> Self
pub const fn new(block: SealedBlock<B>, data: T) -> Self
Creates a new sealed block with associated data.
Sourcepub const fn block(&self) -> &SealedBlock<B>
pub const fn block(&self) -> &SealedBlock<B>
Returns the sealed block.
Sourcepub fn split(self) -> (SealedBlock<B>, T)
pub fn split(self) -> (SealedBlock<B>, T)
Consumes the type and returns its components.
Sourceimpl<B: Block, T> SealedBlockWith<B, Option<T>>
impl<B: Block, T> SealedBlockWith<B, Option<T>>
Sourcepub const fn from_block(block: SealedBlock<B>) -> Self
pub const fn from_block(block: SealedBlock<B>) -> Self
Creates a sealed block without associated data.
Methods from Deref<Target = SealedBlock<B>>
Sourcepub fn clone_block(&self) -> B
pub fn clone_block(&self) -> B
Clones the wrapped block.
Sourcepub fn rlp_length(&self) -> usize
pub fn rlp_length(&self) -> usize
Returns the length of the block.
Sourcepub fn senders(&self) -> Result<Vec<Address>, RecoveryError>
pub fn senders(&self) -> Result<Vec<Address>, RecoveryError>
Recovers all senders from the transactions in the block.
Returns an error if any of the transactions fail to recover the sender.
Sourcepub fn num_hash(&self) -> BlockNumHash
pub fn num_hash(&self) -> BlockNumHash
Return the number hash tuple.
Sourcepub fn block_with_parent(&self) -> BlockWithParent
pub fn block_with_parent(&self) -> BlockWithParent
Return a BlockWithParent for this header.
Sourcepub fn sealed_header(&self) -> &SealedHeader<B::Header>
pub fn sealed_header(&self) -> &SealedHeader<B::Header>
Returns the Sealed header.
Sourcepub fn sealed_header_ref(&self) -> SealedHeader<&B::Header>
pub fn sealed_header_ref(&self) -> SealedHeader<&B::Header>
Returns the wrapped SealedHeader<B::Header> as SealedHeader<&B::Header>.
Sourcepub fn clone_sealed_header(&self) -> SealedHeader<B::Header>
pub fn clone_sealed_header(&self) -> SealedHeader<B::Header>
Clones the wrapped header and returns a SealedHeader sealed with the hash.
Sourcepub fn blob_versioned_hashes_iter(&self) -> impl Iterator<Item = &B256> + '_
pub fn blob_versioned_hashes_iter(&self) -> impl Iterator<Item = &B256> + '_
Returns an iterator over all blob versioned hashes from the block body.
Sourcepub fn transaction_count(&self) -> usize
pub fn transaction_count(&self) -> usize
Returns the number of transactions in the block.
Sourcepub fn ensure_transaction_root_valid(&self) -> Result<(), GotExpected<B256>>
pub fn ensure_transaction_root_valid(&self) -> Result<(), GotExpected<B256>>
Ensures that the transaction root in the block header is valid.
The transaction root is the Keccak 256-bit hash of the root node of the trie structure populated with each transaction in the transactions list portion of the block.
Returns
Returns Ok(()) if the calculated transaction root matches the one stored in the header,
indicating that the transactions in the block are correctly represented in the trie.
Returns Err(error) if the transaction root validation fails, providing a GotExpected
error containing the calculated and expected roots.