pub struct SealedHeader<H = Header> { /* private fields */ }Expand description
Seals the header with the block hash.
This type uses lazy sealing to avoid hashing the header until it is needed:
SealedHeader::new_unhashed creates a sealed header without hashing the header.
SealedHeader::new creates a sealed header with the corresponding block hash.
SealedHeader::hash computes the hash if it has not been computed yet.
Implementations
Sourceimpl<H> SealedHeader<H>
impl<H> SealedHeader<H>
Sourcepub fn new_unhashed(header: H) -> Self
pub fn new_unhashed(header: H) -> Self
Creates the sealed header without hashing the header.
Sourcepub fn new(header: H, hash: BlockHash) -> Self
pub fn new(header: H, hash: BlockHash) -> Self
Creates the sealed header with the corresponding block hash.
Sourcepub fn clone_header(&self) -> Hwhere
H: Clone,
pub fn clone_header(&self) -> Hwhere
H: Clone,
Clone the header.
Sourcepub fn into_header(self) -> H
pub fn into_header(self) -> H
Consumes the type and returns the wrapped header.
Sourcepub fn sealed_ref(&self) -> SealedHeader<&H>
pub fn sealed_ref(&self) -> SealedHeader<&H>
Converts from &SealedHeader<H> to SealedHeader<&H>.
Sourceimpl<H: Sealable> SealedHeader<H>
impl<H: Sealable> SealedHeader<H>
Sourcepub fn hash_ref(&self) -> &BlockHash
pub fn hash_ref(&self) -> &BlockHash
Returns the block hash.
Note: if the hash has not been computed yet, this will compute the hash:
Sealable::hash_slow.
Sourcepub fn split(self) -> (H, BlockHash)
pub fn split(self) -> (H, BlockHash)
This is the inverse of Self::seal_slow which returns the raw header and hash.
Sourceimpl<H: Sealable> SealedHeader<&H>
impl<H: Sealable> SealedHeader<&H>
Sourcepub fn cloned(self) -> SealedHeader<H>where
H: Clone,
pub fn cloned(self) -> SealedHeader<H>where
H: Clone,
Maps a SealedHeader<&H> to a SealedHeader<H> by cloning the header.
Sourceimpl<H: BlockHeader + Sealable> SealedHeader<H>
impl<H: BlockHeader + Sealable> SealedHeader<H>
Sourcepub fn num_hash(&self) -> BlockNumHash
pub fn num_hash(&self) -> BlockNumHash
Return the number hash tuple.
Sourcepub fn block_with_parent(&self) -> BlockWithParent
pub fn block_with_parent(&self) -> BlockWithParent
Return a BlockWithParent for this header.