pub trait MaybeCompact { }Available on non-crate feature
reth-codec only.Expand description
Noop. Helper trait that would require database encoding implementation if reth-codec feature
were enabled.
pub trait MaybeCompact { }reth-codec only.Noop. Helper trait that would require database encoding implementation if reth-codec feature
were enabled.